@@ -67,8 +67,6 @@ struct _GeditDocumentPrivate
 	GeditUndoManager *undo_manager;
 };
 
-static gint current_max_untitled_num = 0;
-
 enum {
 	NAME_CHANGED,
 	SAVED,
@@ -106,6 +104,52 @@ static gboolean gedit_document_auto_save_timeout (GeditDocument *doc);
 static GtkTextBufferClass *parent_class 	= NULL;
 static guint document_signals[LAST_SIGNAL] 	= { 0 };
 
+static GHashTable* allocated_untitled_numbers = NULL;
+
+static gint gedit_document_get_untitled_number (void);
+static void gedit_document_release_untitled_number (gint n);
+
+static gint
+gedit_document_get_untitled_number (void)
+{
+	gint i = 1;
+	
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	if (allocated_untitled_numbers == NULL)
+		allocated_untitled_numbers = g_hash_table_new (NULL, NULL);
+
+	g_return_val_if_fail (allocated_untitled_numbers != NULL, -1);
+	
+	while (TRUE)
+	{
+		if (g_hash_table_lookup (allocated_untitled_numbers, GINT_TO_POINTER (i)) == NULL)
+		{
+			g_hash_table_insert (allocated_untitled_numbers, 
+					     GINT_TO_POINTER (i),
+					     GINT_TO_POINTER (i));
+	
+			return i;
+		}
+		
+		++i;
+	}					
+}
+
+static void
+gedit_document_release_untitled_number (gint n)
+{
+	gboolean ret;
+	
+	g_return_if_fail (allocated_untitled_numbers != NULL);
+
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	ret = g_hash_table_remove (allocated_untitled_numbers, GINT_TO_POINTER (n));
+	g_return_if_fail (ret);	
+}
+
+
 
 GType
 gedit_document_get_type (void)
@@ -283,20 +327,19 @@ gedit_document_finalize (GObject *object)
 	if (document->priv->auto_save_timeout > 0)
 		g_source_remove (document->priv->auto_save_timeout);
 
+	if (document->priv->untitled_number > 0)
+	{
+		g_return_if_fail (document->priv->uri == NULL);
+		gedit_document_release_untitled_number (
+				document->priv->untitled_number);
+	}
+
 	if (document->priv->uri)
     	{
 		g_free (document->priv->uri);
       		document->priv->uri = NULL;
-
-		if (current_max_untitled_num == document->priv->untitled_number)
-			--current_max_untitled_num;
-    	}
-	else
-	{
-		if (current_max_untitled_num == document->priv->untitled_number)
-			--current_max_untitled_num;	
-	}
-
+	}		
+	
 	if (document->priv->last_searched_text)
 		g_free (document->priv->last_searched_text);
 
@@ -328,7 +371,9 @@ gedit_document_new (void)
 	document = GEDIT_DOCUMENT (g_object_new (GEDIT_TYPE_DOCUMENT, NULL));
 
 	g_return_val_if_fail (document->priv != NULL, NULL);
-  	document->priv->untitled_number = ++current_max_untitled_num;
+  	
+	document->priv->untitled_number = gedit_document_get_untitled_number ();
+	g_return_val_if_fail (document->priv->untitled_number > 0, NULL);
 
 	return document;
 }
@@ -338,9 +383,9 @@ gedit_document_new (void)
  * @uri: the URI of the file that has to be loaded
  * @error: return location for error or NULL
  * 
- * Creates a new untitled document.
+ * Creates a new document.
  *
- * Return value: a new untitled document
+ * Return value: a new document
  **/
 GeditDocument*
 gedit_document_new_with_uri (const gchar *uri, GError **error)
@@ -598,8 +643,9 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 				if (conv_error != NULL)
 					g_error_free (conv_error);
 
-				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GNOME_VFS_ERROR_WRONG_FORMAT,
-				_("Invalid UTF-8 data encountered reading file"));
+				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, 
+					     GEDIT_ERROR_INVALID_UTF8_DATA,
+					     _("Invalid UTF-8 data"));
 				
 				if (converted_file_contents != NULL)
 					g_free (converted_file_contents);
@@ -704,8 +750,8 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 	{
 		if (!g_utf8_validate (file_contents->str, file_contents->len, NULL))
 		{
-			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GNOME_VFS_ERROR_WRONG_FORMAT,
-				_("Invalid UTF-8 data encountered reading file"));
+			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GEDIT_ERROR_INVALID_UTF8_DATA,
+				_("Invalid UTF-8 data"));
 			g_string_free (file_contents, TRUE);
 			return FALSE;
 		}
@@ -762,7 +808,12 @@ gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 		g_free (doc->priv->uri);
 			
 	doc->priv->uri = g_strdup (uri);
-	doc->priv->untitled_number = 0;
+
+	if (doc->priv->untitled_number > 0)
+	{
+		gedit_document_release_untitled_number (doc->priv->untitled_number);
+		doc->priv->untitled_number = 0;
+	}
 	
 	g_signal_emit (G_OBJECT (doc), document_signals[NAME_CHANGED], 0);
 }
@@ -863,6 +914,9 @@ gedit_document_save_a_copy_as (GeditDocument* doc, const gchar *uri, GError **er
 	return ret;
 }
 
+/* FIXME: define new ERROR_CODE and remove the error 
+ * strings from here -- Paolo
+ */
 static gboolean	
 gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	       gboolean create_backup_copy, GError **error)
@@ -882,9 +936,23 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 
 	if (!gedit_utils_uri_has_file_scheme (uri))
 	{
-		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, EROFS, g_strerror (EROFS));
-		return FALSE;
-	
+		gchar *error_message;
+		gchar *scheme_string = gnome_vfs_x_uri_get_scheme (uri);
+                
+		if (scheme_string != NULL)
+		{
+			error_message = g_strdup_printf (
+				_("gedit cannot handle %s: locations in write mode."),
+                               	scheme_string);
+
+			g_free (scheme_string);
+		}
+		else
+			error_message = g_strdup_printf (
+				_("gedit cannot handle this kind of location in write mode."));
+
+		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, EROFS, error_message);
+		return FALSE;	
 	}
 	
 	fname = gnome_vfs_x_format_uri_for_display (uri);
@@ -898,8 +966,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
       			if (errno != ENOENT)
 			{
 				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno,
-					"Cannot create the backup copy of the file.\n"
-					"The file has not been saved.");
+					_("Could not create the backup copy of the file."));
 		
 				if (fname != NULL) 
 					g_free (fname);
@@ -918,8 +985,8 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	if ((fname == NULL) || ((file = fopen (fname, "w")) == NULL))
 	{
 		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno,
-			"Make sure that the path you provided exists,\n"
-			"and that you have the appropriate write permissions.");
+			_("Make sure that the path you provided exists, "
+			  "and that you have the appropriate write permissions."));
 						
 		res = FALSE;
 
@@ -950,7 +1017,24 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	
 	if (fputs (chars, file) == EOF || fclose (file) == EOF)
 	{
-		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno, g_strerror (errno));
+		gchar *errmsg;
+
+		switch (errno)
+		{
+			case ENOSPC: 
+				errmsg = g_strdup_printf (
+					_("There is no enough disk space to save the file.\n"
+					  "Please, free some space on the disk and try again."));
+				break;
+			case EFBIG:
+				errmsg = g_strdup_printf (
+					_("The file is too big."));
+				break;
+			default:
+				errmsg = g_strdup_printf (" "); 
+		}
+
+		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno, errmsg);
 		g_free (chars);
 
 		res = FALSE;
@@ -1078,7 +1162,7 @@ gedit_document_revert (GeditDocument *doc,  GError **error)
 
 	if (gedit_document_is_untitled (doc))
 	{
-		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GNOME_VFS_ERROR_GENERIC,
+		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GEDIT_ERROR_UNTITLED,
 			_("It is not possible to revert an Untitled document"));
 		return FALSE;
 	}
