@@ -5,6 +5,7 @@
  *
  * Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
  * Copyright (C) 2000, 2001 Chema Celorio, Paolo Maggi 
+ * Copyright (C) 2002  Paolo Maggi 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +24,7 @@
  */
  
 /*
- * Modified by the gedit Team, 1998-2001. See the AUTHORS file for a 
+ * Modified by the gedit Team, 1998-2002. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  */
@@ -36,6 +37,8 @@
 #include <libgnomeui/libgnomeui.h>
 #include <libgnomevfs/gnome-vfs.h>
 
+#include <string.h>
+
 #include "gedit-mdi.h"
 #include "gedit-mdi-child.h"
 #include "gedit2.h"
@@ -81,6 +84,12 @@ static void gedit_mdi_child_state_changed_handler (GeditMDIChild *child);
 
 static void gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BonoboMDI *mdi);
 
+static void gedit_mdi_view_menu_item_toggled_handler (
+			BonoboUIComponent           *ui_component,
+			const char                  *path,
+			Bonobo_UIComponent_EventType type,
+			const char                  *state,
+			BonoboWindow                *win);
 
 static BonoboMDIClass *parent_class = NULL;
 
@@ -128,7 +137,7 @@ gedit_mdi_init (GeditMDI  *mdi)
 {
 	gedit_debug (DEBUG_MDI, "START");
 
-	bonobo_mdi_construct (BONOBO_MDI (mdi), "gedit2", "gedit", 
+	bonobo_mdi_construct (BONOBO_MDI (mdi), "gedit-2", "gedit", 
 			      gedit_settings->mdi_tabs_position);
 	
 	mdi->priv = g_new0 (GeditMDIPrivate, 1);
@@ -211,7 +220,8 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 {
 	GtkWidget *widget;
 	BonoboControl *control;
-	
+	BonoboUIComponent *ui_component;
+
 	static GtkTargetEntry drag_types[] =
 	{
 		{ "text/uri-list", 0, 0 },
@@ -220,7 +230,10 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	static gint n_drag_types = sizeof (drag_types) / sizeof (drag_types [0]);
 
 	gedit_debug (DEBUG_MDI, "");
-
+	
+	ui_component = bonobo_mdi_get_ui_component_from_window (win);
+	g_return_if_fail (ui_component != NULL);
+	
 	/* Drag and drop support */
 	gtk_drag_dest_set (GTK_WIDGET (win),
 			   GTK_DEST_DEFAULT_MOTION |
@@ -240,7 +253,7 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	gtk_widget_set_size_request (widget, 150, 10);	
 	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (widget), FALSE);
 	
-	bonobo_ui_component_object_set (bonobo_mdi_get_ui_component_from_window (win),
+	bonobo_ui_component_object_set (ui_component,
 		       			"/status/CursorPosition",
 					BONOBO_OBJREF (control),
 					NULL);
@@ -255,7 +268,7 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	
 	gtk_widget_set_size_request (widget, 80, 10);
 	
-	bonobo_ui_component_object_set (bonobo_mdi_get_ui_component_from_window (win),
+	bonobo_ui_component_object_set (ui_component,
 		       			"/status/OverwriteMode",
 					BONOBO_OBJREF (control),
 					NULL);
@@ -270,6 +283,35 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	/* Set the toolbar style according to prefs */
 	gedit_mdi_set_app_toolbar_style (win);
 		
+	/* Add listener fo the view menu */
+	bonobo_ui_component_add_listener (ui_component, "ViewToolbar", 
+			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
+			(gpointer)win);
+	bonobo_ui_component_add_listener (ui_component, "ViewStatusbar", 
+			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
+			(gpointer)win);
+
+	bonobo_ui_component_add_listener (ui_component, "ToolbarSystem", 
+			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
+			(gpointer)win);
+	bonobo_ui_component_add_listener (ui_component, "ToolbarIcon", 
+			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
+			(gpointer)win);
+	bonobo_ui_component_add_listener (ui_component, "ToolbarIconText", 
+			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
+			(gpointer)win);
+	bonobo_ui_component_add_listener (ui_component, "ToolbarTooltips", 
+			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
+			(gpointer)win);
+
+	bonobo_ui_component_add_listener (ui_component, "StatusBarCursorPosition", 
+			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
+			(gpointer)win);
+	bonobo_ui_component_add_listener (ui_component, "StatusBarOverwriteMode", 
+			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
+			(gpointer)win);
+
+
 	/* Set the window prefs. */
 	gtk_window_set_default_size (GTK_WINDOW (win), 
 			gedit_settings->window_width, 
@@ -286,6 +328,87 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	gedit_plugins_engine_update_plugins_ui (win, TRUE);
 }
 
+static void
+gedit_mdi_view_menu_item_toggled_handler (
+			BonoboUIComponent           *ui_component,
+			const char                  *path,
+			Bonobo_UIComponent_EventType type,
+			const char                  *state,
+			BonoboWindow                *win)
+{
+	gboolean s;
+
+	gedit_debug (DEBUG_MDI, "%s toggled to '%s'", path, state);
+
+	s = (strcmp (state, "1") == 0);
+
+	if ((strcmp (path, "ViewToolbar") == 0) &&
+	    (s != gedit_settings->toolbar_visible))
+	{
+		gedit_settings->toolbar_visible = s;
+		gedit_mdi_set_app_toolbar_style (win);
+
+		return;
+	}
+
+	if ((strcmp (path, "ViewStatusbar") == 0) &&
+	    (s != gedit_settings->statusbar_visible))
+	{
+		gedit_settings->statusbar_visible = s;
+		gedit_mdi_set_app_statusbar_style (win);
+	}
+
+	if (s && (strcmp (path, "ToolbarSystem") == 0) &&
+	    (gedit_settings->toolbar_buttons_style != GEDIT_TOOLBAR_SYSTEM))
+	{		
+		gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_SYSTEM;
+		gedit_mdi_set_app_toolbar_style (win);
+
+		return;
+	}
+
+	if (s && (strcmp (path, "ToolbarIcon") == 0) &&
+	    (gedit_settings->toolbar_buttons_style != GEDIT_TOOLBAR_ICONS))
+	{		
+		gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS;
+		gedit_mdi_set_app_toolbar_style (win);
+
+		return;
+	}
+
+	if (s && (strcmp (path, "ToolbarIconText") == 0) &&
+	    (gedit_settings->toolbar_buttons_style != GEDIT_TOOLBAR_ICONS_AND_TEXT))
+	{		
+		gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS_AND_TEXT;
+		gedit_mdi_set_app_toolbar_style (win);
+
+		return;
+	}
+
+	if ((strcmp (path, "ToolbarTooltips") == 0) &&
+	    (s != gedit_settings->toolbar_view_tooltips))
+	{
+		gedit_settings->toolbar_view_tooltips = s;
+		gedit_mdi_set_app_toolbar_style (win);
+
+		return;
+	}
+
+	if ((strcmp (path, "StatusBarCursorPosition") == 0) &&
+	    (s != gedit_settings->statusbar_view_cursor_position))
+	{
+		gedit_settings->statusbar_view_cursor_position = s;
+		gedit_mdi_set_app_statusbar_style (win);
+	}
+
+	if ((strcmp (path, "StatusBarOverwriteMode") == 0) &&
+	    (s != gedit_settings->statusbar_view_overwrite_mode))
+	{
+		gedit_settings->statusbar_view_overwrite_mode = s;
+		gedit_mdi_set_app_statusbar_style (win);
+	}
+}
+
 static void 
 gedit_mdi_drag_data_received_handler (GtkWidget *widget, GdkDragContext *context, 
 		                      gint x, gint y, GtkSelectionData *selection_data, 
@@ -335,25 +458,50 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 			
 	ui_component = bonobo_mdi_get_ui_component_from_window (win);
 	g_return_if_fail (ui_component != NULL);
-	
-	if (!gedit_settings->toolbar_visible)
-	{
-		bonobo_ui_component_set_prop (
-			ui_component, "/Toolbar",
-			"hidden", "1", NULL);
-		return;
-	}
-	
+			
 	bonobo_ui_component_freeze (ui_component, NULL);
 
-	bonobo_ui_component_set_prop (
-		ui_component, "/Toolbar", "hidden", "0", NULL);
+	/* Updated view menu */
+	gedit_menus_set_verb_state (ui_component, 
+				    "/commands/ViewToolbar",
+				    gedit_settings->toolbar_visible);
 
+	gedit_menus_set_verb_sensitive (ui_component, 
+				        "/commands/ToolbarSystem",
+				        gedit_settings->toolbar_visible);
+	gedit_menus_set_verb_sensitive (ui_component, 
+				        "/commands/ToolbarIcon",
+				        gedit_settings->toolbar_visible);
+	gedit_menus_set_verb_sensitive (ui_component, 
+				        "/commands/ToolbarIconText",
+				        gedit_settings->toolbar_visible);
+	gedit_menus_set_verb_sensitive (ui_component, 
+				        "/commands/ToolbarTooltips",
+				        gedit_settings->toolbar_visible);
+
+	gedit_menus_set_verb_state (ui_component, 
+				    "/commands/ToolbarSystem",
+				    gedit_settings->toolbar_buttons_style == GEDIT_TOOLBAR_SYSTEM);
+
+	gedit_menus_set_verb_state (ui_component, 
+				    "/commands/ToolbarIcon",
+				    gedit_settings->toolbar_buttons_style == GEDIT_TOOLBAR_ICONS);
+
+	gedit_menus_set_verb_state (ui_component, 
+				    "/commands/ToolbarIconText",
+				    gedit_settings->toolbar_buttons_style == GEDIT_TOOLBAR_ICONS_AND_TEXT);
+
+	gedit_menus_set_verb_state (ui_component, 
+				    "/commands/ToolbarTooltips",
+				    gedit_settings->toolbar_view_tooltips);
+
+	
+	/* Actually update toolbar style */
 	bonobo_ui_component_set_prop (
 		ui_component, "/Toolbar",
 		"tips", gedit_settings->toolbar_view_tooltips ? "1" : "0",
 		NULL);
-
+	
 	switch (gedit_settings->toolbar_buttons_style)
 	{
 		case GEDIT_TOOLBAR_SYSTEM:
@@ -380,23 +528,31 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 				bonobo_ui_component_set_prop (
 					ui_component, "/Toolbar", "look", "icons", NULL);
 			}
+	
 			break;
+			
 		case GEDIT_TOOLBAR_ICONS:
 			gedit_debug (DEBUG_MDI, "GEDIT: ICONS");
 			bonobo_ui_component_set_prop (
 				ui_component, "/Toolbar", "look", "icon", NULL);
-
+			
 			break;
+			
 		case GEDIT_TOOLBAR_ICONS_AND_TEXT:
 			gedit_debug (DEBUG_MDI, "GEDIT: ICONS_AND_TEXT");
 			bonobo_ui_component_set_prop (
 				ui_component, "/Toolbar", "look", "both", NULL);
+			
 			break;
 		default:
 			goto error;
-		break;
+			break;
 	}
 	
+	bonobo_ui_component_set_prop (
+			ui_component, "/Toolbar",
+			"hidden", gedit_settings->toolbar_visible ? "0":"1", NULL);
+
  error:
 	bonobo_ui_component_thaw (ui_component, NULL);
 }
@@ -414,13 +570,39 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 	ui_component = bonobo_mdi_get_ui_component_from_window (win);
 	g_return_if_fail (ui_component != NULL);
 
+	bonobo_ui_component_freeze (ui_component, NULL);
+
+	/* Update menu */
+	gedit_menus_set_verb_state (ui_component, 
+				    "/commands/ViewStatusbar",
+				    gedit_settings->statusbar_visible);
+
+	gedit_menus_set_verb_sensitive (ui_component, 
+				    "/commands/StatusBarCursorPosition",
+				    gedit_settings->statusbar_visible);
+
+	gedit_menus_set_verb_sensitive (ui_component, 
+				    "/commands/StatusBarOverwriteMode",
+				    gedit_settings->statusbar_visible);
+
+	gedit_menus_set_verb_state (ui_component, 
+				    "/commands/StatusBarCursorPosition",
+				    gedit_settings->statusbar_view_cursor_position);
+
+	gedit_menus_set_verb_state (ui_component, 
+				    "/commands/StatusBarOverwriteMode",
+				    gedit_settings->statusbar_view_overwrite_mode);
+
+	
+	/* Actually update status bar style */
 	bonobo_ui_component_set_prop (
 		ui_component, "/status",
 		"hidden", gedit_settings->statusbar_visible ? "0" : "1",
 		NULL);
 
 	cp = GTK_WIDGET (g_object_get_data (G_OBJECT (win), "CursorPosition"));
-	g_return_if_fail (cp != NULL);
+	if (cp == NULL)
+		goto error;
 
 	if (gedit_settings->statusbar_view_cursor_position)
 		gtk_widget_show (cp);
@@ -428,7 +610,8 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 		gtk_widget_hide (cp);
 
 	om = GTK_WIDGET (g_object_get_data (G_OBJECT (win), "OverwriteMode"));
-	g_return_if_fail (om != NULL);
+	if (om == NULL)
+		goto error;
 
 	if (gedit_settings->statusbar_view_overwrite_mode)
 		gtk_widget_show (om);
@@ -443,10 +626,14 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 	if (!gedit_settings->statusbar_view_cursor_position &&
 	    !gedit_settings->statusbar_view_overwrite_mode)
 		bonobo_ui_component_set_prop (
-			ui_component, "/status/item", "resize_grip", "1", NULL);
+			ui_component, "/status/main", "resize_grip", "1", NULL);
 	else
 		bonobo_ui_component_set_prop (
-			ui_component, "/status/item", "resize_grip", "0", NULL);
+			ui_component, "/status/main", "resize_grip", "0", NULL);
+
+error:
+	bonobo_ui_component_thaw (ui_component, NULL);
+	
 }
 
 static void 
@@ -546,7 +733,7 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_QUESTION,
 				GTK_BUTTONS_NONE,
-				_("Do you want to save the changes you made to the document ``%s''? \n\n"
+				_("Do you want to save the changes you made to the document \"%s\"? \n\n"
 				  "Your changes will be lost if you don't save them."),
 				fname);
 
@@ -566,7 +753,8 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 			else
 				exiting = FALSE;
 		}
-			
+
+#if 0		
 		if (exiting)
 			gedit_dialog_add_button (GTK_DIALOG (msgbox),
 					_("_Don't quit"), GTK_STOCK_CANCEL,
@@ -575,6 +763,9 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 			gedit_dialog_add_button (GTK_DIALOG (msgbox),
 					_("_Don't close"), GTK_STOCK_CANCEL,
                 	             	GTK_RESPONSE_CANCEL);
+#endif
+		
+		gtk_dialog_add_button (GTK_DIALOG (msgbox), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 
 		gtk_dialog_add_button (GTK_DIALOG (msgbox),
 			       	GTK_STOCK_SAVE,
@@ -613,12 +804,12 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 	/*
 	if (close)
 	{
-		gtk_signal_disconnect_by_func (GTK_OBJECT (child), 
-				       G_CALLBACK (gedit_mdi_child_state_changed_handler),
-				       NULL);
-		gtk_signal_disconnect_by_func (GTK_OBJECT (child), 
-				       G_CALLBACK (gedit_mdi_child_undo_redo_state_changed_handler),
-				       NULL);
+		g_signal_handlers_disconnect_by_func (child, 
+						      G_CALLBACK (gedit_mdi_child_state_changed_handler),
+						      NULL);
+		g_signal_handlers_disconnect_by_func (GTK_OBJECT (child), 
+						      G_CALLBACK (gedit_mdi_child_undo_redo_state_changed_handler),
+						      NULL);
 	}
 	*/
 	
@@ -717,6 +908,31 @@ void gedit_mdi_view_changed_handler (BonoboMDI *mdi, GtkWidget *old_view)
 	gedit_view_set_overwrite_mode_statusbar (active_view, status);
 }
 
+void 
+gedit_mdi_clear_active_window_statusbar (GeditMDI *mdi)
+{
+	gpointer status;
+	BonoboWindow *win;
+
+	win = bonobo_mdi_get_active_window (BONOBO_MDI (mdi));
+	if (win == NULL)
+		return;
+
+	status = g_object_get_data (G_OBJECT (win), "CursorPosition");	
+	g_return_if_fail (status != NULL);
+	g_return_if_fail (GTK_IS_STATUSBAR (status));
+
+	/* clear any previous message, underflow is allowed */
+	gtk_statusbar_pop (GTK_STATUSBAR (status), 0); 
+
+	status = g_object_get_data (G_OBJECT (win), "OverwriteMode");	
+	g_return_if_fail (status != NULL);
+	g_return_if_fail (GTK_IS_STATUSBAR (status));
+
+	/* clear any previous message, underflow is allowed */
+	gtk_statusbar_pop (GTK_STATUSBAR (status), 0); 
+}
+
 void 
 gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 {
@@ -823,7 +1039,9 @@ gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BonoboMDI *mdi)
 void
 gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi)
 {
+#if 0
 	GList *windows;		
+#endif
 	GList *children;
 	GdkColor background;
 	GdkColor text;
@@ -833,6 +1051,7 @@ gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi)
 
 	gedit_debug (DEBUG_MDI, "");
 
+#if 0
 	windows = bonobo_mdi_get_windows (BONOBO_MDI (mdi));
 
 	while (windows != NULL)
@@ -853,7 +1072,7 @@ gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi)
 
 		windows = windows->next;
 	}
-
+#endif
 	children = bonobo_mdi_get_children (BONOBO_MDI (mdi));
 
 	if (gedit_settings->use_default_colors)
